import logging
from datetime import datetime

from django.contrib.auth.decorators import login_required
from django.http import JsonResponse
from django.urls import reverse
from django.utils import timezone

from web.api_views.serialization_utils import serialize_datetime, location_to_str, flying_team_to_str
from web.models import Appointment
from web.views.appointment import APPOINTMENT_LIST_GENERIC, APPOINTMENT_LIST_UNFINISHED, APPOINTMENT_LIST_APPROACHING
from web.views.notifications import get_filter_locations, \
    get_today_midnight_date, \
    get_unfinished_appointments

logger = logging.getLogger(__name__)


@login_required
def get_appointments(request, type, min_date, max_date):
    if type == APPOINTMENT_LIST_GENERIC:
        result = Appointment.objects.filter(location__in=get_filter_locations(request.user),
                                            )
    elif type == APPOINTMENT_LIST_UNFINISHED:
        result = get_unfinished_appointments(request.user)
    elif type == APPOINTMENT_LIST_APPROACHING:
        result = Appointment.objects.filter(
            datetime_when__gt=get_today_midnight_date(),
            location__in=get_filter_locations(request.user),
            status=Appointment.APPOINTMENT_STATUS_SCHEDULED
        ).order_by("datetime_when")
    else:
        raise TypeError("Unknown query type: " + type)

    if min_date is not None:
        min_date = datetime.strptime(min_date, "%Y-%m-%d").replace(tzinfo=timezone.now().tzinfo)
        result = result.filter(datetime_when__gt=min_date)

    if max_date is not None:
        max_date = datetime.strptime(max_date, "%Y-%m-%d").replace(tzinfo=timezone.now().tzinfo)
        result = result.filter(datetime_when__lt=max_date)
    return result.order_by("datetime_when")


@login_required
def appointments(request, type):
    # id of the query from dataTable: https://datatables.net/manual/server-side
    draw = int(request.GET.get("draw", "-1"))

    start = int(request.GET.get("start", "0"))
    length = int(request.GET.get("length", "10"))

    min_date = request.GET.get("start_date", None)
    max_date = request.GET.get("end_date", None)

    if min_date is not None:
        length = 1000000000

    all_appointments = get_appointments(request, type, min_date, max_date)

    count = all_appointments.count()

    sliced_subjects = all_appointments[start:(start + length)]

    result_appointments = sliced_subjects

    count_filtered = all_appointments.count()

    data = []
    for appointment in result_appointments:
        data.append(serialize_appointment(appointment))

    return JsonResponse({
        "draw": draw,
        "recordsTotal": count,
        "recordsFiltered": count_filtered,
        "data": data,
    })


def serialize_appointment(appointment):
    subject_string = ""
    nd_number = screening_number = phone_numbers = appointment_types = None
    if appointment.visit is not None:
        title = "Visit " + str(appointment.visit.visit_number)
        study_subject = appointment.visit.subject
        subject_string = study_subject.subject.last_name + " " + study_subject.subject.first_name
        nd_number = study_subject.nd_number
        screening_number = study_subject.screening_number
        phone_numbers = ", ".join(filter(None,
                                         [study_subject.subject.phone_number, study_subject.subject.phone_number_2,
                                          study_subject.subject.phone_number_3]))
        appointment_types = ", ".join(
            [unicode(appointment_type) for appointment_type in appointment.appointment_types.all()])
    else:
        title = appointment.comment

    appointment_type = ", ".join([appointment_type.code for appointment_type in appointment.appointment_types.all()])
    time = serialize_datetime(appointment.datetime_when)
    until = serialize_datetime(appointment.datetime_until())

    location = location_to_str(appointment.location)
    flying_team = flying_team_to_str(appointment.flying_team)

    result = {
        "subject": subject_string,
        "title": title,
        "nd_number": nd_number,
        "screening_number": screening_number,
        "phone_number": phone_numbers,
        "appointment_types": appointment_types,
        "type": appointment_type,
        "datetime_when": time,
        "datetime_until": until,
        "comment": appointment.comment,
        "color": appointment.color(),
        "id": appointment.id,
        "location": location,
        "flying_team": flying_team,
        "url": reverse('web.views.appointment_edit', kwargs={'id': str(appointment.id)})
    }
    return result
