# coding=utf-8

import datetime

from django.contrib.auth.models import User
from django.test import Client
from django.test import TestCase
from django.urls import reverse

from web.models import AppointmentTypeLink
from web.tests.functions import create_study_subject, create_worker, create_visit, create_appointment, \
    create_appointment_type, create_get_suffix, create_flying_team
from web.views.appointment import APPOINTMENT_LIST_GENERIC, APPOINTMENT_LIST_APPROACHING, APPOINTMENT_LIST_UNFINISHED
from web.views.notifications import get_today_midnight_date


class TestAppointmentApi(TestCase):
    def setUp(self):
        self.study_subject = create_study_subject()
        self.client = Client()
        username = 'piotr'
        password = 'top_secret'
        self.user = User.objects.create_user(
            username=username, email='jacob@bla', password=password)
        self.worker = create_worker(self.user)
        self.client.login(username=username, password=password)

    def test_appointments_valid(self):
        place = "Some new flying team location"
        name = "Piotrek"
        self.study_subject.subject.first_name = name
        self.study_subject.subject.save()
        visit = create_visit(self.study_subject)
        appointment = create_appointment(visit)
        appointment.flying_team = create_flying_team(place=place)
        appointment.save()
        appointment2 = create_appointment(visit, get_today_midnight_date())
        appointment2.visit = None

        appointment_type = create_appointment_type()
        appointment2.save()
        AppointmentTypeLink.objects.create(appointment=appointment2, appointment_type=appointment_type)

        url = reverse('web.api.appointments', kwargs={'type': APPOINTMENT_LIST_GENERIC})
        response = self.client.get(url)

        self.assertEqual(response.status_code, 200)
        self.assertTrue(name in response.content)
        self.assertTrue(place in response.content)

    def test_flying_team_location_in_appointments(self):
        place = "Some new flying team location"
        visit = create_visit(self.study_subject)
        appointment = create_appointment(visit)
        appointment.flying_team = create_flying_team(place=place)
        appointment.save()

        url = reverse('web.api.appointments', kwargs={'type': APPOINTMENT_LIST_GENERIC})
        response = self.client.get(url)

        self.assertEqual(response.status_code, 200)
        self.assertTrue(place in response.content)

    def test_appointments_approaching(self):
        name = "Piotrek"
        self.study_subject.subject.first_name = name
        self.study_subject.subject.save()
        visit = create_visit(self.study_subject)
        create_appointment(visit, get_today_midnight_date() + datetime.timedelta(days=2))

        url = reverse('web.api.appointments', kwargs={'type': APPOINTMENT_LIST_APPROACHING})
        response = self.client.get(url)

        self.assertEqual(response.status_code, 200)
        self.assertTrue(name in response.content)

    def test_appointments_unfinished(self):
        name = "Piotrek"
        self.study_subject.subject.first_name = name
        self.study_subject.subject.save()
        visit = create_visit(self.study_subject)
        create_appointment(visit, get_today_midnight_date() + datetime.timedelta(days=-12))

        url = reverse('web.api.appointments', kwargs={'type': APPOINTMENT_LIST_UNFINISHED})
        response = self.client.get(url)

        self.assertEqual(response.status_code, 200)
        self.assertTrue(name in response.content)

    def test_get_calendar_appointments(self):
        name = "Peter"
        self.study_subject.subject.first_name = name
        self.study_subject.subject.save()
        visit = create_visit(self.study_subject)
        appointment = create_appointment(visit, get_today_midnight_date())
        appointment.save()

        AppointmentTypeLink.objects.create(appointment=appointment, appointment_type=create_appointment_type())

        params = {
            "start_date": (get_today_midnight_date() + datetime.timedelta(days=2)).strftime("%Y-%m-%d"),
            "end_date": (get_today_midnight_date() + datetime.timedelta(days=3)).strftime("%Y-%m-%d"),
        }
        url = ("%s" + create_get_suffix(params)) % reverse('web.api.appointments',
                                                           kwargs={'type': APPOINTMENT_LIST_GENERIC})
        response = self.client.get(url)

        self.assertEqual(response.status_code, 200)
        self.assertFalse(name in response.content)

        params["start_date"] = (get_today_midnight_date() + datetime.timedelta(days=-2)).strftime("%Y-%m-%d")
        url = ("%s" + create_get_suffix(params)) % reverse('web.api.appointments',
                                                           kwargs={'type': APPOINTMENT_LIST_GENERIC})
        response = self.client.get(url)

        self.assertEqual(response.status_code, 200)
        self.assertTrue(name in response.content)
