import datetime
import logging

from django.urls import reverse

from web.forms import SubjectAddForm, SubjectEditForm, StudySubjectAddForm, StudySubjectEditForm
from web.models import MailTemplate, StudySubject
from web.models.constants import SEX_CHOICES_MALE, SUBJECT_TYPE_CHOICES_CONTROL, SUBJECT_TYPE_CHOICES_PATIENT, \
    COUNTRY_AFGHANISTAN_ID, COUNTRY_OTHER_ID, MAIL_TEMPLATE_CONTEXT_SUBJECT
from web.tests import LoggedInWithWorkerTestCase
from web.tests.functions import create_study_subject, create_visit, create_appointment, get_test_location, \
    create_language, get_resource_path, get_test_study
from web.views.notifications import get_today_midnight_date

logger = logging.getLogger(__name__)


class SubjectsViewTests(LoggedInWithWorkerTestCase):
    def setUp(self):
        super(SubjectsViewTests, self).setUp()
        self.study_subject = create_study_subject()
        self.study = get_test_study()

    def test_render_subjects_add(self):
        self.worker.save()

        response = self.client.get(reverse('web.views.subject_add'))
        self.assertEqual(response.status_code, 200)

    def test_render_subject_edit(self):
        response = self.client.get(reverse('web.views.subject_edit', kwargs={'id': self.study_subject.id}))
        self.assertEqual(response.status_code, 200)

    def test_render_subject_edit_with_mail_templates(self):
        language = create_language(name="German")
        template_name = "german_template"
        template_file = get_resource_path('upcoming_appointment_FR.docx')
        self.study_subject.subject.default_written_communication_language = language
        self.study_subject.subject.save()

        MailTemplate(name=template_name, language=language, context=MAIL_TEMPLATE_CONTEXT_SUBJECT,
                     template_file=template_file).save()

        response = self.client.get(reverse('web.views.subject_edit', kwargs={'id': self.study_subject.id}))
        self.assertEqual(response.status_code, 200)
        self.assertTrue(template_name in response.content)

    def test_render_subject_visit_details(self):
        visit = create_visit(self.study_subject)
        create_appointment(visit)

        response = self.client.get(reverse('web.views.subject_visit_details', kwargs={'id': self.study_subject.id}))
        self.assertEqual(response.status_code, 200)
        self.assertFalse("Add visit" in response.content)

    def test_render_subject_visit_details_without_visit(self):
        response = self.client.get(reverse('web.views.subject_visit_details', kwargs={'id': self.study_subject.id}))
        self.assertEqual(response.status_code, 200)
        self.assertTrue("Add visit" in response.content)

    def test_save_subject_edit_when_resigned_without_reason(self):
        form_data = self.create_edit_form_data_for_study_subject()

        form_data['subject-dead'] = "True"
        form_data['study_subject-resigned'] = "True"
        response = self.client.post(
            reverse('web.views.subject_edit', kwargs={'id': self.study_subject.id}), data=form_data)

        self.assertEqual(response.status_code, 200)
        self.assertTrue("Resign reason cannot be empty" in response.content)

    def test_save_subject_edit(self):
        form_data = self.create_edit_form_data_for_study_subject()

        form_data['subject-dead'] = "True"
        form_data['study_subject-resigned'] = "True"
        form_data['study_subject-resign_reason'] = "doesn't want to participate"
        response = self.client.post(
            reverse('web.views.subject_edit', kwargs={'id': self.study_subject.id}), data=form_data)

        self.assertEqual(response.status_code, 302)
        updated_study_subject = StudySubject.objects.filter(id=self.study_subject.id)[0]
        self.assertTrue(updated_study_subject.subject.dead)
        self.assertTrue(updated_study_subject.resigned)

    def test_save_subject_edit_with_continue(self):
        form_data = self.create_edit_form_data_for_study_subject()
        form_data['_continue'] = True

        response = self.client.post(reverse('web.views.subject_edit', kwargs={'id': self.study_subject.id}),
                                    data=form_data)

        self.assertEqual(response.status_code, 302)
        self.assertTrue("edit" in response.url)

    def create_edit_form_data_for_study_subject(self):
        form_study_subject = StudySubjectEditForm(instance=self.study_subject, prefix="study_subject")
        form_subject = SubjectEditForm(instance=self.study_subject.subject, prefix="subject")
        form_data = {}
        for key, value in form_study_subject.initial.items():
            if value is not None:
                form_data['study_subject-{}'.format(key)] = value
        for key, value in form_subject.initial.items():
            if value is not None:
                form_data['subject-{}'.format(key)] = value
        return form_data

    def create_add_form_data_for_study_subject(self):
        form_study_subject = StudySubjectAddForm(prefix="study_subject", user=self.user, study=self.study)
        form_subject = SubjectAddForm(prefix="subject")
        form_data = {}
        for key, value in form_study_subject.initial.items():
            if value is not None:
                form_data['study_subject-{}'.format(key)] = value
        for key, value in form_subject.initial.items():
            if value is not None:
                form_data['subject-{}'.format(key)] = value
        self.add_valid_form_data_for_subject_add(form_data)
        return form_data

    def test_subjects_add_2(self):
        form_data = self.create_add_form_data_for_study_subject()

        form_data["study_subject-type"] = SUBJECT_TYPE_CHOICES_CONTROL
        form_data["study_subject-default_location"] = get_test_location().id
        response = self.client.post(reverse('web.views.subject_add'), data=form_data)
        self.assertEqual(response.status_code, 302)
        response = self.client.get(response.url)
        self.assertContains(response, "Subject created")

        subject = StudySubject.objects.all().order_by("-id")[0]
        self.assertEqual("L-001", subject.screening_number,
                         "prefix should start by L" +
                         " as default location prefix is not defined and subject type is control")

    def add_valid_form_data_for_subject_add(self, form_data):
        form_data["subject-country"] = COUNTRY_AFGHANISTAN_ID
        form_data["subject-first_name"] = "John"
        form_data["subject-last_name"] = "Doe"
        form_data["subject-sex"] = SEX_CHOICES_MALE
        form_data["study_subject-type"] = SUBJECT_TYPE_CHOICES_PATIENT
        form_data["study_subject-subject"] = self.study_subject.id

        # TODO remove after refactoring
        form_data["study_subject-country"] = COUNTRY_AFGHANISTAN_ID
        form_data["study_subject-first_name"] = "John"
        form_data["study_subject-last_name"] = "Doe"

    def test_subjects_add_patient(self):
        form_data = self.create_add_form_data_for_study_subject()

        form_data["study_subject-default_location"] = get_test_location().id
        response = self.client.post(reverse('web.views.subject_add'), data=form_data)
        self.assertEqual(response.status_code, 302)
        response = self.client.get(response.url)
        self.assertContains(response, "Subject created")

        subject = StudySubject.objects.all().order_by("-id")[0]
        self.assertEqual("P-001", subject.screening_number,
                         "prefix should start by P" +
                         " as default location prefix is not defined and subject type is patient")

    def test_subjects_add_invalid(self):
        form_data = self.create_add_form_data_for_study_subject()
        form_data["study_subject-type"] = SUBJECT_TYPE_CHOICES_CONTROL
        form_data["study_subject-default_location"] = get_test_location().id

        form_data["subject-country"] = COUNTRY_OTHER_ID
        response = self.client.post(reverse('web.views.subject_add'), data=form_data)

        self.assertTrue("Invalid data" in response.content)

    def test_subjects_add_with_prefixed_location(self):
        form_data = self.create_add_form_data_for_study_subject()
        form_data["study_subject-type"] = SUBJECT_TYPE_CHOICES_CONTROL

        self.add_valid_form_data_for_subject_add(form_data)
        location = get_test_location()
        location.prefix = 'X'
        location.save()
        form_data["study_subject-default_location"] = get_test_location().id
        response = self.client.post(reverse('web.views.subject_add'), data=form_data)
        self.assertEqual(response.status_code, 302)
        response = self.client.get(response.url)
        self.assertContains(response, "Subject created")

        subject = StudySubject.objects.all().order_by("-id")[0]
        self.assertEqual("X-001", subject.screening_number,
                         "prefix should start by X as default location prefix is equal to 'X'")

    def test_render_subjects(self):
        response = self.client.get(reverse('web.views.subjects'))
        self.assertEqual(response.status_code, 200)

    def test_render_subjects_with_no_visit(self):
        response = self.client.get(reverse('web.views.subject_no_visits'))
        self.assertEqual(response.status_code, 200)

    def test_render_subjects_require_contact(self):
        self.study_subject.datetime_contact_reminder = get_today_midnight_date() + datetime.timedelta(days=-1)

        response = self.client.get(reverse('web.views.subject_require_contact'))
        self.assertEqual(response.status_code, 200)
