# coding=utf-8
import csv

import django_excel as excel
from django.contrib.auth.decorators import login_required
from django.http import HttpResponse

from notifications import get_today_midnight_date
from . import e500_error, wrap_response
from ..models import Subject, StudySubject, Appointment


@login_required
def export_to_csv(request, data_type="subjects"):
    # Create the HttpResponse object with the appropriate CSV header.
    response = HttpResponse(content_type='text/csv')
    response['Content-Disposition'] = 'attachment; filename="' + data_type + '-' + get_today_midnight_date().strftime(
        "%Y-%m-%d") + '.csv"'

    if data_type == "subjects":
        data = get_subjects_as_array()
    elif data_type == "appointments":
        data = get_appointments_as_array()
    else:
        return e500_error(request)
    writer = csv.writer(response, quotechar=str(u'"'), quoting=csv.QUOTE_ALL)
    for row in data:
        writer.writerow([s.encode("utf-8") for s in row])

    return response


@login_required
def export_to_excel(request, data_type="subjects"):
    filename = data_type + '-' + get_today_midnight_date().strftime("%Y-%m-%d") + ".xls"
    if data_type == "subjects":
        data = get_subjects_as_array()
    elif data_type == "appointments":
        data = get_appointments_as_array()
    else:
        return e500_error(request)

    response = excel.make_response_from_array(data, 'xls', file_name=filename)
    response['Content-Disposition'] = 'attachment; filename="' + filename + '"'

    return response


class CustomField:
    def __init__(self, dictionary):
        for k, v in dictionary.items():
            setattr(self, k, v)


DROP_OUT_FIELD = CustomField({'verbose_name': "DROP OUT", 'name': "custom-drop-out"})


def get_subjects_as_array():
    result = []
    subject_fields = get_default_subject_fields()

    field_names = []
    for field in subject_fields:
        field_names.append(field.verbose_name)

    result.append(field_names)

    subjects = StudySubject.objects.order_by('-subject__last_name')
    for subject in subjects:
        row = subject_to_row_for_fields(subject, subject_fields)
        result.append([unicode(s).replace("\n", ";").replace("\r", ";") for s in row])
    return result


def subject_to_row_for_fields(study_subject, subject_fields):
    row = []
    for field in subject_fields:
        if field == DROP_OUT_FIELD:
            if not study_subject.resigned:
                cell = False
            else:
                finished_appointments = Appointment.objects.filter(visit__subject=study_subject).filter(
                    status=Appointment.APPOINTMENT_STATUS_FINISHED).count()
                if finished_appointments > 0:
                    cell = True
                else:
                    cell = False
        else:
            if hasattr(study_subject, field.name):
                cell = getattr(study_subject, field.name)
            elif hasattr(study_subject.subject, field.name):
                cell = getattr(study_subject.subject, field.name)
            if cell is None:
                cell = ""
        row.append(cell)
    return row


def get_default_subject_fields():
    subject_fields = []
    for field in Subject._meta.fields:
        if field.name != "ID":
            subject_fields.append(field)
    for field in StudySubject._meta.fields:
        if field.name != "ID" and field.name != "subject":
            subject_fields.append(field)
    subject_fields.append(DROP_OUT_FIELD)
    return subject_fields


def get_appointments_as_array():
    result = []
    appointments_fields = []
    for field in Appointment._meta.fields:
        if field.name != "visit" and field.name != "id" and \
                        field.name != "worker_assigned" and field.name != "appointment_types" and \
                        field.name != "room" and field.name != "flying_team":
            appointments_fields.append(field)

    field_names = ['ND number', 'Family name', 'Name', 'Visit']
    for field in appointments_fields:
        field_names.append(field.verbose_name)

    result.append(field_names)

    appointments = Appointment.objects.order_by('-datetime_when')

    for appointment in appointments:
        if appointment.visit is not None:
            row = [appointment.visit.subject.nd_number, appointment.visit.subject.subject.last_name,
                   appointment.visit.subject.subject.first_name, appointment.visit.visit_number]
        else:
            row = ["---", "---", "---", "---"]
        for field in appointments_fields:
            row.append(getattr(appointment, field.name))
        type_string = ""
        for appointment_type in appointment.appointment_types.all():
            type_string += appointment_type.code + ","
        row.append(type_string)
        result.append([unicode(s).replace("\n", ";").replace("\r", ";") for s in row])
    return result


def export(request):
    return wrap_response(request, 'export/index.html', {})
